@@ -14,9 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -31,8 +29,9 @@
 #include "gtkmarshalers.h"
 #include "gtkintl.h"
 #include "gtktextbuffer.h"
+#include "gtkselectionprivate.h"
 #include "gtkquartz.h"
-#include "gtkalias.h"
+#include "quartz/gdkquartz-gtk-only.h"
 
 enum {
   OWNER_CHANGE,
@@ -129,7 +128,7 @@ static GtkClipboard *clipboard_peek       (GdkDisplay       *display,
 
 /*  pasteboardChangedOwner is not called immediately, and it's not called
  *  reliably. It is somehow documented in the apple api docs, but the docs
- *  suck and don't really give clear instructions. Therefore we track
+ *  suck and don’t really give clear instructions. Therefore we track
  *  changeCount in several places below and clear the clipboard if it
  *  changed.
  */
@@ -199,13 +198,18 @@ gtk_clipboard_class_init (GtkClipboardClass *class)
 
   class->owner_change = gtk_clipboard_owner_change;
 
+  /**
+   * GtkClipboard::owner-change:
+   * @clipboard:
+   * @event: (type GdkEventOwnerChange):
+   */
   clipboard_signals[OWNER_CHANGE] =
     g_signal_new (I_("owner-change"),
 		  G_TYPE_FROM_CLASS (gobject_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkClipboardClass, owner_change),
 		  NULL, NULL,
-		  _gtk_marshal_VOID__BOXED,
+		  NULL,
 		  G_TYPE_NONE, 1,
 		  GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
 }
@@ -253,22 +257,55 @@ clipboard_display_closed (GdkDisplay   *display,
   g_object_unref (clipboard);
 }
 
+/**
+ * gtk_clipboard_get_for_display:
+ * @display:
+ * @selection:
+ *
+ * Returns: (transfer none):
+ */
 GtkClipboard *
 gtk_clipboard_get_for_display (GdkDisplay *display,
 			       GdkAtom     selection)
 {
   g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
-  g_return_val_if_fail (!display->closed, NULL);
+  g_return_val_if_fail (!gdk_display_is_closed (display), NULL);
 
   return clipboard_peek (display, selection, FALSE);
 }
 
+/**
+ * gtk_clipboard_get:
+ * @selection:
+ *
+ * Returns: (transfer none):
+ */
 GtkClipboard *
 gtk_clipboard_get (GdkAtom selection)
 {
   return gtk_clipboard_get_for_display (gdk_display_get_default (), selection);
 }
 
+/**
+ * gtk_clipboard_get_default:
+ * @display: the #GdkDisplay for which the clipboard is to be retrieved.
+ *
+ * Returns the default clipboard object for use with cut/copy/paste menu items
+ * and keyboard shortcuts.
+ *
+ * Return value: (transfer none): the default clipboard object.
+ *
+ * Since: 3.16
+ **/
+GtkClipboard *
+gtk_clipboard_get_default (GdkDisplay *display)
+{
+  g_return_val_if_fail (display != NULL, NULL);
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
+
+  return gtk_clipboard_get_for_display (display, GDK_SELECTION_CLIPBOARD);
+}
+
 static void
 clipboard_owner_destroyed (gpointer data)
 {
@@ -333,7 +370,6 @@ gtk_clipboard_set_contents (GtkClipboard         *clipboard,
 			    gpointer              user_data,
 			    gboolean              have_owner)
 {
-  GtkClipboardOwner *owner;
   NSSet *types;
   NSAutoreleasePool *pool;
 
@@ -369,26 +405,35 @@ gtk_clipboard_set_contents (GtkClipboard         *clipboard,
    */
   if (user_data && user_data == clipboard->user_data)
     {
-      owner = [clipboard->owner retain];
-
-      owner->setting_same_owner = TRUE;
+      clipboard->owner->setting_same_owner = TRUE;
       clipboard->change_count = [clipboard->pasteboard declareTypes: [types allObjects]
-                                                              owner: owner];
-      owner->setting_same_owner = FALSE;
+                                                              owner: clipboard->owner];
+      clipboard->owner->setting_same_owner = FALSE;
     }
   else
     {
-      owner = [[GtkClipboardOwner alloc] initWithClipboard:clipboard];
+      GtkClipboardOwner *new_owner;
 
+      /* We do not set the new owner on clipboard->owner immediately,
+       * because declareTypes could (but not always does -- see also the
+       * comment at pasteboardChangedOwner above) cause clipboard_unset
+       * to be called, which releases clipboard->owner.
+       */
+      new_owner = [[GtkClipboardOwner alloc] initWithClipboard:clipboard];
       clipboard->change_count = [clipboard->pasteboard declareTypes: [types allObjects]
-                                                              owner: owner];
+                                                              owner: new_owner];
+
+      /* In case pasteboardChangedOwner was not triggered, check to see
+       * whether the previous owner still needs to be released.
+       */
+      if (clipboard->owner)
+        [clipboard->owner release];
+      clipboard->owner = new_owner;
     }
 
-  [owner release];
   [types release];
   [pool release];
 
-  clipboard->owner = owner;
   clipboard->user_data = user_data;
   clipboard->have_owner = have_owner;
   if (have_owner)
@@ -403,6 +448,15 @@ gtk_clipboard_set_contents (GtkClipboard         *clipboard,
   return TRUE;
 }
 
+/**
+ * gtk_clipboard_set_with_data: (skip)
+ * @clipboard:
+ * @targets: (array length=n_targets):
+ * @n_targets:
+ * @get_func: (scope async):
+ * @clear_func: (scope async):
+ * @user_data:
+ */
 gboolean
 gtk_clipboard_set_with_data (GtkClipboard          *clipboard,
 			     const GtkTargetEntry  *targets,
@@ -420,6 +474,14 @@ gtk_clipboard_set_with_data (GtkClipboard          *clipboard,
 				     FALSE);
 }
 
+/**
+ * gtk_clipboard_set_with_owner: (skip)
+ * @clipboard:
+ * @targets: (array length=n_targets):
+ * @get_func: (scope async):
+ * @clear_func: (scope async):
+ * @owner:
+ */
 gboolean
 gtk_clipboard_set_with_owner (GtkClipboard          *clipboard,
 			      const GtkTargetEntry  *targets,
@@ -438,6 +500,12 @@ gtk_clipboard_set_with_owner (GtkClipboard          *clipboard,
 				     TRUE);
 }
 
+/**
+ * gtk_clipboard_get_owner:
+ * @clipboard:
+ *
+ * Returns: (transfer none):
+ */
 GObject *
 gtk_clipboard_get_owner (GtkClipboard *clipboard)
 {
@@ -478,6 +546,8 @@ clipboard_unset (GtkClipboard *clipboard)
   g_free (clipboard->storable_targets);
   clipboard->storable_targets = NULL;
 
+  if (clipboard->owner)
+    [clipboard->owner release];
   clipboard->owner = NULL;
   clipboard->get_func = NULL;
   clipboard->clear_func = NULL;
@@ -504,8 +574,16 @@ void
 gtk_clipboard_clear (GtkClipboard *clipboard)
 {
   clipboard_unset (clipboard);
-
-  [clipboard->pasteboard declareTypes:nil owner:nil];
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1060
+  if (gdk_quartz_osx_version() >= GDK_OSX_SNOW_LEOPARD)
+    [clipboard->pasteboard clearContents];
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
+  else
+#endif
+#endif
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
+    [clipboard->pasteboard declareTypes:nil owner:nil];
+#endif
 }
 
 static void
@@ -596,6 +674,13 @@ gtk_clipboard_set_image (GtkClipboard *clipboard,
   gtk_target_list_unref (list);
 }
 
+/**
+ * gtk_clipboard_request_contents:
+ * @clipboard:
+ * @target:
+ * @callback: (scope async):
+ * @user_data:
+ */
 void
 gtk_clipboard_request_contents (GtkClipboard            *clipboard,
 				GdkAtom                  target,
@@ -611,6 +696,12 @@ gtk_clipboard_request_contents (GtkClipboard            *clipboard,
   gtk_selection_data_free (data);
 }
 
+/**
+ * gtk_clipboard_request_text:
+ * @clipboard:
+ * @callback: (scope async):
+ * @user_data:
+ */
 void
 gtk_clipboard_request_text (GtkClipboard                *clipboard,
 			    GtkClipboardTextReceivedFunc callback,
@@ -623,6 +714,13 @@ gtk_clipboard_request_text (GtkClipboard                *clipboard,
   g_free (data);
 }
 
+/**
+ * gtk_clipboard_request_rich_text:
+ * @clipboard:
+ * @buffer:
+ * @callback: (scope async):
+ * @user_data:
+ */
 void
 gtk_clipboard_request_rich_text (GtkClipboard                    *clipboard,
                                  GtkTextBuffer                   *buffer,
@@ -633,6 +731,15 @@ gtk_clipboard_request_rich_text (GtkClipboard                    *clipboard,
 }
 
 
+/**
+ * gtk_clipboard_wait_for_rich_text:
+ * @clipboard:
+ * @buffer:
+ * @format: (out):
+ * @length: (out):
+ *
+ * Returns: (nullable):
+ */
 guint8 *
 gtk_clipboard_wait_for_rich_text (GtkClipboard  *clipboard,
                                   GtkTextBuffer *buffer,
@@ -643,6 +750,12 @@ gtk_clipboard_wait_for_rich_text (GtkClipboard  *clipboard,
   return NULL;
 }
 
+/**
+ * gtk_clipboard_request_image:
+ * @clipboard:
+ * @callback: (scope async):
+ * @user_data:
+ */
 void
 gtk_clipboard_request_image (GtkClipboard                  *clipboard,
 			     GtkClipboardImageReceivedFunc  callback,
@@ -656,6 +769,12 @@ gtk_clipboard_request_image (GtkClipboard                  *clipboard,
     g_object_unref (pixbuf);
 }
 
+/**
+ * gtk_clipboard_request_uris:
+ * @clipboard:
+ * @callback: (scope async):
+ * @user_data:
+ */
 void
 gtk_clipboard_request_uris (GtkClipboard                *clipboard,
 			    GtkClipboardURIReceivedFunc  callback,
@@ -668,6 +787,12 @@ gtk_clipboard_request_uris (GtkClipboard                *clipboard,
   g_strfreev (uris);
 }
 
+/**
+ * gtk_clipboard_request_targets:
+ * @clipboard:
+ * @callback: (scope async):
+ * @user_data:
+ */
 void
 gtk_clipboard_request_targets (GtkClipboard                *clipboard,
 			       GtkClipboardTargetsReceivedFunc callback,
@@ -681,7 +806,13 @@ gtk_clipboard_request_targets (GtkClipboard                *clipboard,
   callback (clipboard, targets, n_targets, user_data);
 }
 
-
+/**
+ * gtk_clipboard_wait_for_contents:
+ * @clipboard:
+ * @target:
+ *
+ * Returns: (nullable):
+ */
 GtkSelectionData *
 gtk_clipboard_wait_for_contents (GtkClipboard *clipboard,
 				 GdkAtom       target)
@@ -735,6 +866,12 @@ gtk_clipboard_wait_for_contents (GtkClipboard *clipboard,
   return selection_data;
 }
 
+/**
+ * gtk_clipboard_wait_for_text:
+ * @clipboard:
+ *
+ * Returns: (nullable):
+ */
 gchar *
 gtk_clipboard_wait_for_text (GtkClipboard *clipboard)
 {
@@ -751,11 +888,16 @@ gtk_clipboard_wait_for_text (GtkClipboard *clipboard)
   return result;
 }
 
+/**
+ * gtk_clipboard_wait_for_image:
+ * @clipboard:
+ *
+ * Returns: (nullable) (transfer full):
+ */
 GdkPixbuf *
 gtk_clipboard_wait_for_image (GtkClipboard *clipboard)
 {
   GdkAtom target = gdk_atom_intern_static_string("image/tiff");
-  int i;
   GtkSelectionData *data;
 
   data = gtk_clipboard_wait_for_contents (clipboard, target);
@@ -770,6 +912,12 @@ gtk_clipboard_wait_for_image (GtkClipboard *clipboard)
   return NULL;
 }
 
+/**
+ * gtk_clipboard_wait_for_uris:
+ * @clipboard:
+ *
+ * Returns: (nullable) (array zero-terminated=1) (transfer full) (element-type utf8):
+ */
 gchar **
 gtk_clipboard_wait_for_uris (GtkClipboard *clipboard)
 {
@@ -789,6 +937,12 @@ gtk_clipboard_wait_for_uris (GtkClipboard *clipboard)
   return NULL;
 }
 
+/**
+ * gtk_clipboard_get_display:
+ * @clipboard:
+ *
+ * Returns: (transfer none):
+ */
 GdkDisplay *
 gtk_clipboard_get_display (GtkClipboard *clipboard)
 {
@@ -867,6 +1021,25 @@ gtk_clipboard_wait_is_uris_available (GtkClipboard *clipboard)
   return result;
 }
 
+/**
+ * gtk_clipboard_wait_for_targets:
+ * @clipboard: a #GtkClipboard
+ * @targets: (out) (array length=n_targets) (transfer container): location
+ *           to store an array of targets. The result stored here must
+ *           be freed with g_free().
+ * @n_targets: (out): location to store number of items in @targets.
+ *
+ * Returns a list of targets that are present on the clipboard, or %NULL
+ * if there aren’t any targets available. The returned list must be
+ * freed with g_free().
+ * This function waits for the data to be received using the main
+ * loop, so events, timeouts, etc, may be dispatched during the wait.
+ *
+ * Returns: %TRUE if any targets are present on the clipboard,
+ *               otherwise %FALSE.
+ *
+ * Since: 2.4
+ */
 gboolean
 gtk_clipboard_wait_for_targets (GtkClipboard  *clipboard,
 				GdkAtom      **targets,
@@ -1027,6 +1200,12 @@ _gtk_clipboard_handle_event (GdkEventOwnerChange *event)
 {
 }
 
+/**
+ * gtk_clipboard_set_can_store:
+ * @clipboard:
+ * @targets: (allow-none) (array length=n_targets):
+ * @n_targets:
+ */
 void
 gtk_clipboard_set_can_store (GtkClipboard         *clipboard,
  			     const GtkTargetEntry *targets,
@@ -1108,8 +1287,20 @@ _gtk_clipboard_store_all (void)
 
       list = list->next;
     }
-  g_slist_free (displays);
 }
 
-#define __GTK_CLIPBOARD_C__
-#include "gtkaliasdef.c"
+/**
+ * gtk_clipboard_get_selection:
+ * @clipboard:
+ *
+ * Returns: the selection
+ *
+ * Since: 3.22
+ */
+GdkAtom
+gtk_clipboard_get_selection (GtkClipboard *clipboard)
+{
+  g_return_val_if_fail (GTK_IS_CLIPBOARD (clipboard), GDK_NONE);
+
+  return clipboard->selection;
+}
